/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 *
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odt.dom.components;

import com.openexchange.office.filter.odf.DLNode;
import com.openexchange.office.filter.odf.components.Component;
import com.openexchange.office.filter.odf.components.ComponentContext;
import com.openexchange.office.filter.odf.components.TextComponent;
import com.openexchange.office.filter.odf.components.TextFieldComponent;
import com.openexchange.office.filter.odf.components.TextLineBreakComponent;
import com.openexchange.office.filter.odf.components.TextTabComponent;
import com.openexchange.office.filter.odf.draw.DrawFrame;
import com.openexchange.office.filter.odf.draw.Shape;
import com.openexchange.office.filter.odt.dom.Annotation;
import com.openexchange.office.filter.odt.dom.AnnotationEnd;
import com.openexchange.office.filter.odt.dom.Text;
import com.openexchange.office.filter.odt.dom.TextField;
import com.openexchange.office.filter.odt.dom.TextLineBreak;
import com.openexchange.office.filter.odt.dom.TextSpan;
import com.openexchange.office.filter.odt.dom.TextTab;

public class TextSpanContext extends ComponentContext {

	public TextSpanContext(ComponentContext parentContext, DLNode<Object> textSpanNode) {
		super(parentContext, textSpanNode);
	}

	@Override
	public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
        final int nextComponentNumber = previousChildComponent != null ? previousChildComponent.getNextComponentNumber() : 0;
        DLNode<Object> nextNode = previousChildContext != null ? previousChildContext.getNode().next : ((TextSpan)getNode().getObject()).getContent().getFirstNode();
		while(nextNode!=null) {
			final Object child = nextNode.getObject();
			if(child instanceof Text) {
				if(!((Text)child).getText().isEmpty()) {
					return new TextComponent(this, nextNode, nextComponentNumber);
				}
			}
			else if(child instanceof TextTab) {
				return new TextTabComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof TextLineBreak) {
				return new TextLineBreakComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof TextField) {
				return new TextFieldComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof DrawFrame) {
				return new FrameRootComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof Shape) {
				return new ShapeRootComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof Annotation) {
				return new AnnotationComponent(this, nextNode, nextComponentNumber);
			}
			else if(child instanceof AnnotationEnd) {
				return new AnnotationEndComponent(this, nextNode, nextComponentNumber);
			}
			nextNode = nextNode.next;
		}
		return null;
	}
}
